package org.foxarmy.barcodescannerforemployees

import android.content.Context
import android.os.Build
import android.util.Log
import androidx.annotation.RequiresApi
import okhttp3.*
import org.foxarmy.barcodescannerforemployees.activities.MainActivity
import org.foxarmy.barcodescannerforemployees.database.AbstractProductDAO
import org.foxarmy.barcodescannerforemployees.database.CategoryDAO
import org.foxarmy.barcodescannerforemployees.database.DBStorageController
import org.foxarmy.barcodescannerforemployees.database.ProductDAO
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Category
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import org.json.JSONObject
import java.io.File
import java.util.concurrent.TimeUnit

class WebSocketClient(private val context: Context, private val server: String) {

    private lateinit var webSocket: WebSocket
    private lateinit var token: String
    private lateinit var currentGroup: String

    fun connect(token: String, currentGroup: String) {
        this.token = token
        this.currentGroup = currentGroup

        val client = OkHttpClient.Builder()
            .connectTimeout(5, TimeUnit.SECONDS)
            .readTimeout(1, TimeUnit.MINUTES)
            .writeTimeout(10, TimeUnit.SECONDS)
            .callTimeout(1, TimeUnit.MINUTES)
            .build()

        val request = Request.Builder()
            .url("wss://$server")
            .build()

        client.newWebSocket(request, object : WebSocketListener() {
            override fun onOpen(webSocket: WebSocket, response: Response) {
                this@WebSocketClient.webSocket = webSocket
                sendMessage("{\"token\":\"$token\",\"currentGroup\":\"$currentGroup\"}")
                keepAlive()
            }

            @RequiresApi(Build.VERSION_CODES.O)
            override fun onMessage(webSocket: WebSocket, text: String) {
                val payload = JSONObject(text)

                val item = payload["item"] as String
                val data = payload["data"] as JSONObject
                val action = payload["action"] as String
                val currentGroup = payload["groupId"] as String

                val dbHelper = DBStorageController(context, currentGroup)
                val abstractProductDAO = AbstractProductDAO(dbHelper)
                val productDAO = ProductDAO(dbHelper)
                val categoryDAO = CategoryDAO(dbHelper)

                when (action) {
                    "create" -> {
                        when (item) {
                            "abstractproduct" -> {
                                val newAbstractProduct = AbstractProduct.createFromJSON(data)

                                val net = Net()
                                net.server = server
                                net.token = token

                                val picturesDir = File(context.filesDir, "pictures")
                                picturesDir.mkdirs()

                                val pictureFile =
                                    File(picturesDir, "${data["image_filename"]}.png")
                                val url = "https://${net.server}/api/abstractproduct/getImage/${currentGroup}/${data["local_id"]}"
                                net.downloadImage(url, pictureFile, {
                                    abstractProductDAO.addAbstractProduct(newAbstractProduct)
                                    (context as MainActivity).updateAll()
                                })
                            }

                            "product" -> {
                                val newProduct = Product.createFromJSON(data)
                                productDAO.insertNewProduct(newProduct)
                                (context as MainActivity).updateAll()
                            }

                            "category" -> {
                                val newCategory = Category.createFromJSON(data)
                                categoryDAO.addCategory(newCategory)
                                (context as MainActivity).updateAll()
                            }
                        }
                    }

                    "update" -> {
                        when (item) {
                            "abstractproduct" -> {
                                val updatedAbstractProduct = AbstractProduct.createFromJSON(data)

                                val net = Net()
                                net.server = server
                                net.token = token

                                val picturesDir = File(context.filesDir, "pictures")
                                picturesDir.mkdirs()

                                val pictureFile =
                                    File(picturesDir, "${data["image_filename"]}.png")
                                val url = "https://${net.server}/api/abstractproduct/getImage/${currentGroup}/${data["local_id"]}"
                                net.downloadImage(url, pictureFile, {
                                    abstractProductDAO.updateAbstractProduct(updatedAbstractProduct)
                                    (context as MainActivity).updateAll()
                                })

                            }
                            "product" -> {
                                val updatedProduct = Product.createFromJSON(data)
                                productDAO.updateProduct(updatedProduct)
                                (context as MainActivity).updateAll()
                            }
                            "category" -> {
                                val updatedCategory = Category.createFromJSON(data)
                                categoryDAO.updateCategory(updatedCategory)
                                (context as MainActivity).updateAll()
                            }
                        }
                    }

                    "delete" -> {
                        val id = data["local_id"] as String

                        when(item) {
                            "abstractproduct" -> {
                                abstractProductDAO.eraseAbstractProduct(id.toInt(), context)
                                (context as MainActivity).updateAll()
                            }
                            "product" -> {
                                productDAO.eraseProduct(id.toInt())
                                (context as MainActivity).updateAll()
                            }
                            "category" -> {
                                categoryDAO.eraseCategory(id.toInt(), context)
                                (context as MainActivity).updateAll()
                            }
                        }
                    }
                }
            }

            override fun onClosing(webSocket: WebSocket, code: Int, reason: String) {
                Log.d("QWERTYUIOP", "Closing ws. Reason: $reason")
//                noInternetConnectionAvailableNotification(context)
                this@WebSocketClient.connect(token, currentGroup)
            }

            override fun onFailure(webSocket: WebSocket, t: Throwable, response: Response?) {
                Log.d("QWERTYUIOP","Connection failed: ${t.message}")
                noInternetConnectionAvailableNotification(context)
            }
        })
    }

    private fun keepAlive() {
        Thread {
            while (true) {
                Thread.sleep(30000)
                webSocket.send("keepalive")
            }
        }.start()
    }

    fun sendMessage(message: String) {
        webSocket.send(message)
    }

    fun close() {
        webSocket.close(1000, "Closing WebSocket connection")
    }
}