package org.foxarmy.barcodescannerforemployees.database

import android.content.ContentValues
import android.provider.BaseColumns
import org.foxarmy.barcodescannerforemployees.dataclasses.Product

class ProductDAO (private val dbHelper: DBStorageController) {

    fun eraseProduct(id: Int) {

        val db = dbHelper.writableDatabase

        db.delete(ProductContract.ProductEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    fun insertNewProduct(product: Product) : Long{

        val db = dbHelper.writableDatabase

        val values = ContentValues().apply {
            if (product.id > 0) put(BaseColumns._ID, product.id)
            put(ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID, product.abstractProductId)
            put(ProductContract.ProductEntry.AMOUNT, product.amount)
            put(ProductContract.ProductEntry.DATE_OF_PRODUCTION, product.dateOfProduction)
            put(ProductContract.ProductEntry.EXPIRY_DATE, product.dateOfExpiry)
        }

        val id = db.insert(ProductContract.ProductEntry.TABLE_NAME, null, values)
        return id
    }

    fun findAmountOfProductsWithExpiryDate(date: Long): Int {

        val db = dbHelper.readableDatabase

        var amount = 0

        val projection = arrayOf(
            ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID,
            ProductContract.ProductEntry.AMOUNT,
            ProductContract.ProductEntry.DATE_OF_PRODUCTION,
        )

        val selection = "${ProductContract.ProductEntry.EXPIRY_DATE} = ?"
        val selectionArgs = arrayOf(date.toString())

        val cursor =
            db.query(ProductContract.ProductEntry.TABLE_NAME, projection, selection, selectionArgs, null, null, null)

        with(cursor) {
            while (moveToNext()) {
                amount++
            }
        }

        return amount
    }

    fun findAllExpiryDates(): Set<Long> {

        val db = dbHelper.readableDatabase

        val dates: MutableSet<Long> = mutableSetOf<Long>()

        val projection = arrayOf(
            ProductContract.ProductEntry.EXPIRY_DATE
        )

        val orderBy = "${ProductContract.ProductEntry.EXPIRY_DATE} ASC"

        val cursor = db.query(ProductContract.ProductEntry.TABLE_NAME, projection, null, null, null, null, orderBy)

        with(cursor) {
            while (moveToNext()) {
                dates.add(getLong(getColumnIndexOrThrow(ProductContract.ProductEntry.EXPIRY_DATE)))
            }
        }

        return dates
    }

    fun updateProduct(product: Product) {

        val db = dbHelper.writableDatabase

        val values = ContentValues().apply {
            put(BaseColumns._ID, product.id)
            put(ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID, product.abstractProductId)
            put(ProductContract.ProductEntry.AMOUNT, product.amount)
            put(ProductContract.ProductEntry.DATE_OF_PRODUCTION, product.dateOfProduction)
            put(ProductContract.ProductEntry.EXPIRY_DATE, product.dateOfExpiry)
        }
        db.update(
            ProductContract.ProductEntry.TABLE_NAME,
            values,
            "${BaseColumns._ID} = ?",
            arrayOf(product.id.toString())
        )
    }

    fun getSortedListOfProducts(selectedSort: Int, filterBy: String, filter: String): List<Product> {

        val db = dbHelper.readableDatabase

        val products = mutableListOf<Product>()

        val projection = arrayOf(
            BaseColumns._ID,
            ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID,
            ProductContract.ProductEntry.AMOUNT,
            ProductContract.ProductEntry.DATE_OF_PRODUCTION,
            ProductContract.ProductEntry.EXPIRY_DATE,
        )

        var orderBy: String = ""

        when (selectedSort) {
            0 -> {
                orderBy =
                    "(SELECT ${AbstractProductContract.AbstractProductEntry.PRODUCT_NAME} FROM ${AbstractProductContract.AbstractProductEntry.TABLE_NAME} WHERE ${BaseColumns._ID} = ${ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID}) ASC"
            }

            1 -> {
                orderBy =
                    "(SELECT ${AbstractProductContract.AbstractProductEntry.CATEGORY} FROM ${AbstractProductContract.AbstractProductEntry.TABLE_NAME} WHERE ${BaseColumns._ID} = ${ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID}) ASC"
            }

            3 -> {
                orderBy = "${ProductContract.ProductEntry.DATE_OF_PRODUCTION} ASC"
            }

            4 -> {
                orderBy = "${ProductContract.ProductEntry.EXPIRY_DATE} ASC"
            }
        }

        var selection: String? = null
        var selectionArgs: Array<String>? = null

        when (filterBy) {
            "expiryDate" -> {
                selection = "${ProductContract.ProductEntry.EXPIRY_DATE} = ?"
                selectionArgs = arrayOf(filter)
            }

            "" -> {}
        }

        val cursor = db.query(ProductContract.ProductEntry.TABLE_NAME, projection, selection, selectionArgs, null, null, orderBy)

        with(cursor) {
            while (moveToNext()) {
                val productId = getInt(getColumnIndexOrThrow(android.provider.BaseColumns._ID))
                val abstractProductId =
                    getInt(getColumnIndexOrThrow(ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID))
                val amount = getInt(getColumnIndexOrThrow(ProductContract.ProductEntry.AMOUNT))
                val dateOfProduction =
                    getLong(getColumnIndexOrThrow(ProductContract.ProductEntry.DATE_OF_PRODUCTION))
                val dateOfExpiry = getLong(getColumnIndexOrThrow(ProductContract.ProductEntry.EXPIRY_DATE))

                val product = Product(productId, abstractProductId, amount, dateOfProduction, dateOfExpiry)

                products.add(product)
            }
        }

        products.sortWith(compareByDescending { it.freshness })

        return products
    }

}